"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockFinder = exports.averageBlockTime = void 0;
var tslib_1 = require("tslib");
var assert_1 = tslib_1.__importDefault(require("assert"));
var lodash_1 = require("lodash");
var TypeGuards_1 = require("./TypeGuards");
var TimeUtils_1 = require("./TimeUtils");
var defaultBlockRange = 120;
var cacheTTL = 60 * 15;
var blockTimes = {};
function averageBlockTime(provider, _a) {
    var _b = _a === void 0 ? {} : _a, latestBlockNumber = _b.latestBlockNumber, blockRange = _b.blockRange;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var chainId, cache, now, _c, _d, firstBlock, lastBlock, average;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0: return [4, provider.getNetwork()];
                case 1:
                    chainId = (_e.sent()).chainId;
                    cache = blockTimes[chainId];
                    now = (0, TimeUtils_1.getCurrentTime)();
                    if ((0, TypeGuards_1.isDefined)(cache) && now < cache.timestamp + cacheTTL) {
                        return [2, { average: cache.average, blockRange: cache.blockRange }];
                    }
                    if (!(latestBlockNumber !== null && latestBlockNumber !== void 0)) return [3, 2];
                    _c = latestBlockNumber;
                    return [3, 4];
                case 2: return [4, provider.getBlockNumber()];
                case 3:
                    _c = (latestBlockNumber = _e.sent());
                    _e.label = 4;
                case 4:
                    _c;
                    blockRange !== null && blockRange !== void 0 ? blockRange : (blockRange = defaultBlockRange);
                    return [4, Promise.all([
                            provider.getBlock(latestBlockNumber - blockRange),
                            provider.getBlock(latestBlockNumber),
                        ])];
                case 5:
                    _d = _e.sent(), firstBlock = _d[0], lastBlock = _d[1];
                    average = (lastBlock.timestamp - firstBlock.timestamp) / blockRange;
                    blockTimes[chainId] = { timestamp: now, average: average, blockRange: blockRange };
                    return [2, { average: average, blockRange: blockRange }];
            }
        });
    });
}
exports.averageBlockTime = averageBlockTime;
function estimateBlocksElapsed(seconds, cushionPercentage, provider) {
    if (cushionPercentage === void 0) { cushionPercentage = 0.0; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var cushionMultiplier, average;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    cushionMultiplier = cushionPercentage + 1.0;
                    return [4, averageBlockTime(provider)];
                case 1:
                    average = (_a.sent()).average;
                    return [2, Math.floor((seconds * cushionMultiplier) / average)];
            }
        });
    });
}
var BlockFinder = (function () {
    function BlockFinder(provider, blocks) {
        if (blocks === void 0) { blocks = []; }
        this.provider = provider;
        this.blocks = blocks;
    }
    BlockFinder.prototype.getBlockForTimestamp = function (timestamp) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var block, initialBlock, cushion, incrementDistance, _a, _b, multiplier, distance, blockNumber, block, index;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        timestamp = Number(timestamp);
                        (0, assert_1.default)(timestamp !== undefined && timestamp !== null, "timestamp must be provided");
                        if (!(this.blocks.length === 0 || this.blocks[this.blocks.length - 1].timestamp < timestamp)) return [3, 2];
                        return [4, this.getLatestBlock()];
                    case 1:
                        block = _c.sent();
                        if (timestamp >= block.timestamp)
                            return [2, block];
                        _c.label = 2;
                    case 2:
                        if (!(this.blocks[0].timestamp > timestamp)) return [3, 7];
                        initialBlock = this.blocks[0];
                        cushion = 1;
                        _b = (_a = Math).max;
                        return [4, estimateBlocksElapsed(initialBlock.timestamp - timestamp, cushion, this.provider)];
                    case 3:
                        incrementDistance = _b.apply(_a, [_c.sent(), 1]);
                        multiplier = 1;
                        _c.label = 4;
                    case 4:
                        distance = multiplier * incrementDistance;
                        blockNumber = Math.max(0, initialBlock.number - distance);
                        return [4, this.getBlock(blockNumber)];
                    case 5:
                        block = _c.sent();
                        if (block.timestamp <= timestamp)
                            return [3, 7];
                        (0, assert_1.default)(blockNumber > 0, "timestamp is before block 0");
                        _c.label = 6;
                    case 6:
                        multiplier++;
                        return [3, 4];
                    case 7:
                        index = (0, lodash_1.sortedIndexBy)(this.blocks, { timestamp: timestamp }, "timestamp");
                        return [2, this.findBlock(this.blocks[index - 1], this.blocks[index], timestamp)];
                }
            });
        });
    };
    BlockFinder.prototype.getLatestBlock = function () {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var block, index;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4, this.provider.getBlock("latest")];
                    case 1:
                        block = _b.sent();
                        index = (0, lodash_1.sortedIndexBy)(this.blocks, block, "number");
                        if (((_a = this.blocks[index]) === null || _a === void 0 ? void 0 : _a.number) !== block.number)
                            this.blocks.splice(index, 0, block);
                        return [2, this.blocks[index]];
                }
            });
        });
    };
    BlockFinder.prototype.getBlock = function (number) {
        var _a, _b;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var index, block;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        index = (0, lodash_1.sortedIndexBy)(this.blocks, { number: number }, "number");
                        if (((_a = this.blocks[index]) === null || _a === void 0 ? void 0 : _a.number) === number)
                            return [2, this.blocks[index]];
                        return [4, this.provider.getBlock(number)];
                    case 1:
                        block = _c.sent();
                        index = (0, lodash_1.sortedIndexBy)(this.blocks, { number: number }, "number");
                        if (((_b = this.blocks[index]) === null || _b === void 0 ? void 0 : _b.number) === number)
                            return [2, this.blocks[index]];
                        this.blocks.splice(index, 0, block);
                        return [2, block];
                }
            });
        });
    };
    BlockFinder.prototype.findBlock = function (_startBlock, _endBlock, timestamp) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a, startBlock, endBlock, totalTimeDifference, totalBlockDistance, blockPercentile, estimatedBlock, newBlock;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = [_startBlock, _endBlock], startBlock = _a[0], endBlock = _a[1];
                        if (endBlock.timestamp === timestamp)
                            return [2, endBlock];
                        if (endBlock.number === startBlock.number + 1)
                            return [2, startBlock];
                        (0, assert_1.default)(endBlock.number !== startBlock.number, "startBlock cannot equal endBlock");
                        (0, assert_1.default)(timestamp < endBlock.timestamp && timestamp > startBlock.timestamp, "timestamp not in between start and end blocks");
                        totalTimeDifference = endBlock.timestamp - startBlock.timestamp;
                        totalBlockDistance = endBlock.number - startBlock.number;
                        blockPercentile = (timestamp - startBlock.timestamp) / totalTimeDifference;
                        estimatedBlock = startBlock.number + Math.round(blockPercentile * totalBlockDistance);
                        return [4, this.getBlock((0, lodash_1.clamp)(estimatedBlock, startBlock.number + 1, endBlock.number - 1))];
                    case 1:
                        newBlock = _b.sent();
                        if (newBlock.timestamp < timestamp) {
                            return [2, this.findBlock(newBlock, endBlock, timestamp)];
                        }
                        else {
                            return [2, this.findBlock(startBlock, newBlock, timestamp)];
                        }
                        return [2];
                }
            });
        });
    };
    return BlockFinder;
}());
exports.BlockFinder = BlockFinder;
//# sourceMappingURL=BlockUtils.js.map