/**
 * This is a superstruct that verifies that a UBA Config is formatted correctly.
 * Specifically, it verifies that the UBA Config is an object with the following
 * keys:
 * - incentivePoolAdjustment: a record of numeric strings to numeric strings
 * - ubaRewardMultiplier: a record of numeric strings to numeric strings
 * - alpha: a record of routing rules to numeric strings
 * - gamma: a record of routing rules to numeric tuples
 * - omega: a record of routing rules to numeric tuples
 * - rebalance: a record of routing rules to objects with the following keys:
 *   - threshold_lower: a numeric string
 *   - threshold_upper: a numeric string
 *   - target_lower: a numeric string
 *   - target_upper: a numeric string
 */
export declare const UBA_CONFIG_ONCHAIN_SCHEMA: import("superstruct").Struct<{
    alpha: Record<string, string> | {
        default: string;
    };
    gamma: Record<string, string[][]>;
    omega: Record<string, string[][]>;
    rebalance: {
        default: {
            threshold_lower?: string | undefined;
            threshold_upper?: string | undefined;
            target_lower?: string | undefined;
            target_upper?: string | undefined;
        };
    } | Record<string, {
        threshold_lower?: string | undefined;
        threshold_upper?: string | undefined;
        target_lower?: string | undefined;
        target_upper?: string | undefined;
    }>;
    incentivePoolAdjustment?: Record<string, string> | undefined;
    ubaRewardMultiplier?: Record<string, string> | undefined;
}, {
    incentivePoolAdjustment: import("superstruct").Struct<Record<string, string> | undefined, null>;
    ubaRewardMultiplier: import("superstruct").Struct<Record<string, string> | undefined, null>;
    alpha: import("superstruct").Struct<Record<string, string> | {
        default: string;
    }, null>;
    gamma: import("superstruct").Struct<Record<string, string[][]>, null>;
    omega: import("superstruct").Struct<Record<string, string[][]>, null>;
    rebalance: import("superstruct").Struct<{
        default: {
            threshold_lower?: string | undefined;
            threshold_upper?: string | undefined;
            target_lower?: string | undefined;
            target_upper?: string | undefined;
        };
    } | Record<string, {
        threshold_lower?: string | undefined;
        threshold_upper?: string | undefined;
        target_lower?: string | undefined;
        target_upper?: string | undefined;
    }>, null>;
}>;
